﻿using System;
using System.Text;
using System.Runtime.InteropServices;
using System.Windows;
using System.Windows.Interop;

namespace SyncFiles
{
    public class FolderBrowserDialog
    {
        #region Public Properties

        public String SelectedFolder { get; set; }
        public String Title { get; set; }
        public String InitialFolder { get; set; }
        public String InitialExpandedFolder { get; set; }
        public String OKButtonText { get; set; }
        public BROWSEINFOW BrowseInformation { get; set; }
        public BrowseInfoFlags BrowserDialogFlags { get; set; }

        #endregion

        public FolderBrowserDialog()
        {
            BrowseInformation = new BROWSEINFOW();
            BrowseInformation.lpszTitle = "Select a folder:";
            BrowseInformation.ulFlags = BrowseInfoFlags.BIF_NEWDIALOGSTYLE;
            BrowseInformation.lpfn = new BrowseCallbackProcedure(BrowseEventHandler);
            BrowseInformation.lParam = IntPtr.Zero;
            BrowseInformation.iImage = -1;
            BrowseInformation.hwndOwner = IntPtr.Zero;
            BrowseInformation.pidlRoot = IntPtr.Zero;
            //BrowseInformation.pszDisplayName = new String(' ', 260);
            BrowseInformation.pszDisplayName = String.Empty;
        }

        public Nullable<bool> ShowDialog(Window owner)
        {
            return ShowWindow(owner);
        }

        private Nullable<bool> ShowWindow(Window owner)
        {
            WindowInteropHelper windowhelper;

            if (owner != null)
            {
                windowhelper = new WindowInteropHelper(owner);
                BrowseInformation.hwndOwner = windowhelper.Handle;
            }

            IntPtr handle = SHBrowseForFolderW(BrowseInformation);

            if (handle !=IntPtr.Zero)
            {
                StringBuilder strBuilderPath = new StringBuilder(260);

                if (SHGetPathFromIDList(handle, strBuilderPath) != false)
                {
                    SelectedFolder = strBuilderPath.ToString();
                    Marshal.FreeCoTaskMem(handle);
                    return true;
                }
            }

            return false;
        }

        private int BrowseEventHandler(IntPtr windowHandle, MessageFromBrowser messageFromBrowser, IntPtr lParam, IntPtr lpData)
        {
            switch (messageFromBrowser)
            {
                case MessageFromBrowser.BFFM_INITIALIZED:
                    {
                        if (!String.IsNullOrEmpty(InitialExpandedFolder))
                            SendMessageW(windowHandle, MessageToBrowser.BFFM_SETEXPANDED, new IntPtr(1), InitialExpandedFolder);
                        else if (!String.IsNullOrEmpty(InitialFolder))
                            SendMessageW(windowHandle, MessageToBrowser.BFFM_SETSELECTIONW, new IntPtr(1), InitialFolder);
                        if (!String.IsNullOrEmpty(OKButtonText))
                            SendMessageW(windowHandle, MessageToBrowser.BFFM_SETOKTEXT, new IntPtr(1), OKButtonText);
                        break;
                    }

                case MessageFromBrowser.BFFM_SELCHANGED:
                    {
                        StringBuilder strBuilderPath = new StringBuilder(260);

                        if (SHGetPathFromIDList(lParam, strBuilderPath) != false)
                           SelectedFolder = strBuilderPath.ToString();
                        break;
                    }

                case MessageFromBrowser.BFFM_VALIDATEFAILEDA:                    
                case MessageFromBrowser.BFFM_VALIDATEFAILEDW:                    
                case MessageFromBrowser.BFFM_IUNKNOWN:
                break;                    
            }

            return 0;
        }

        public delegate int BrowseCallbackProcedure(IntPtr hwnd, MessageFromBrowser uMsg, IntPtr lParam, IntPtr lpData);

        [Flags]
        public enum BrowseInfoFlags : uint
        {
            BIF_None = 0x0000,
            BIF_RETURNONLYFSDIRS = 0x0001,
            BIF_DONTGOBELOWDOMAIN = 0x0002,
            BIF_STATUSTEXT = 0x0004, 
            BIF_RETURNFSANCESTORS = 0x0008,
            BIF_EDITBOX = 0x0010,    
            BIF_VALIDATE = 0x0020,   
            BIF_NEWDIALOGSTYLE = 0x0040,
            BIF_USENEWUI = BIF_NEWDIALOGSTYLE | BIF_EDITBOX,
            BIF_BROWSEINCLUDEURLS = 0x0080,
            BIF_UAHINT = 0x0100,           
            BIF_NONEWFOLDERBUTTON = 0x0200,
            BIF_NOTRANSLATETARGETS = 0x0400,
            BIF_BROWSEFORCOMPUTER = 0x1000, 
            BIF_BROWSEFORPRINTER = 0x2000, 
            BIF_BROWSEINCLUDEFILES = 0x4000,
            BIF_SHAREABLE = 0x8000
        }

        [StructLayout(LayoutKind.Sequential, CharSet = CharSet.Unicode)]
        public class BROWSEINFOW
        {
            public IntPtr hwndOwner;
            public IntPtr pidlRoot;
            public String pszDisplayName;
            public String lpszTitle;
            public BrowseInfoFlags ulFlags;
            public BrowseCallbackProcedure lpfn;
            public IntPtr lParam;
            public int iImage;            
        }

        public enum MessageFromBrowser : uint
        {
            BFFM_INITIALIZED = 1,
            BFFM_SELCHANGED = 2,
            BFFM_VALIDATEFAILEDA = 3,
            BFFM_VALIDATEFAILEDW = 4,
            BFFM_IUNKNOWN = 5
        }

        public enum MessageToBrowser : uint
        {
            WM_USER = 0x0400,            
            BFFM_SETSTATUSTEXTA = WM_USER + 100,
            BFFM_ENABLEOK = WM_USER + 101,
            BFFM_SETSELECTIONA = WM_USER + 102,
            BFFM_SETSELECTIONW = WM_USER + 103,
            BFFM_SETSTATUSTEXTW = WM_USER + 104,
            BFFM_SETOKTEXT = WM_USER + 105,
            BFFM_SETEXPANDED = WM_USER + 106
        }

        [DllImport("shell32.dll")]
        private static extern IntPtr SHBrowseForFolderW([MarshalAs(UnmanagedType.LPStruct), In, Out] BROWSEINFOW bi);
        [DllImport("shell32.dll")]
        private static extern bool SHGetPathFromIDList(IntPtr pidl, StringBuilder path);
        [DllImport("user32.dll")]
        public static extern IntPtr SendMessageW(IntPtr hWnd, uint Msg, IntPtr wParam, IntPtr lParam);
        [DllImport("user32.dll")]
        public static extern IntPtr SendMessageW(IntPtr hWnd, MessageToBrowser msg, IntPtr wParam, [MarshalAs(UnmanagedType.LPWStr)] string str);
    }
}